@@ -81,10 +81,6 @@ class Worker:
             else:
                 name = function.__qualname__
 
-            assert asyncio.iscoroutinefunction(
-                function
-            ), f"{function} is not a coroutine"
-
             self.functions[name] = function
 
     async def start(self):
@@ -205,9 +201,8 @@ class Worker:
             if self.before_process:
                 await self.before_process(context)
 
-            task = asyncio.create_task(
-                self.functions[job.function](context, **(job.kwargs or {}))
-            )
+            function = ensure_coroutine_function(self.functions[job.function])
+            task = asyncio.create_task(function(context, **(job.kwargs or {})))
             self.job_task_contexts[job] = {"task": task, "aborted": False}
             result = await asyncio.wait_for(task, job.timeout)
             await job.finish(Status.COMPLETE, result=result)
@@ -240,6 +235,19 @@ class Worker:
             new_task.add_done_callback(self._process)
 
 
+def ensure_coroutine_function(func):
+    if asyncio.iscoroutinefunction(func):
+        return func
+
+    async def wrapped(*args, **kwargs):
+        loop = asyncio.get_running_loop()
+        return await loop.run_in_executor(
+            executor=None, func=lambda: func(*args, **kwargs)
+        )
+
+    return wrapped
+
+
 def import_settings(settings):
     import importlib
 
@@ -249,7 +257,7 @@ def import_settings(settings):
     return getattr(module, name)
 
 
-def start(settings, web=False, port=8080):
+def start(settings, web=False, additional_web_settings=None, port=8080):
     settings = import_settings(settings)
 
     if "queue" not in settings:
@@ -259,15 +267,21 @@ def start(settings, web=False, port=8080):
     worker = Worker(**settings)
 
     if web:
-        import aiohttp
-        from saq.web import app
+        import aiohttp.web
+        from saq.web import create_app
+
+        additional_web_settings = additional_web_settings or []
+        web_settings = [settings] + [
+            import_settings(s) for s in additional_web_settings
+        ]
+        queues = [s["queue"] for s in web_settings if s.get("queue")]
 
         async def shutdown(_app):
             await worker.stop()
-            await worker.queue.disconnect()
 
+        app = create_app(queues)
         app.on_shutdown.append(shutdown)
-        app["queue"] = worker.queue
+
         loop.create_task(worker.start())
         aiohttp.web.run_app(app, port=port, loop=loop)
     else:
